#!/usr/bin/env python
# -*- coding: utf-8 -*-

import base64
import logging
import ast
import ConfigParser

_logger = logging.getLogger(__name__)


class HfCrypto(object):
    """!
    Klasse zum Ver- und Entschlüsseln von Heizdateien.
    """

    def __init__(self):
        """!
        Initialisiert den HfDecryptor.
        """

        _logger.debug("HfDecryptor init")
        self._name = "HfDecryptor"

        self._key = '1xqar02ufaKLTayyMZkLqbZhqXodP2LYDl53oE28cKQ='

    @staticmethod
    def _encode(key, clear):
        enc = []
        for i in range(len(clear)):
            key_c = key[i % len(key)]
            enc_c = chr((ord(clear[i]) + ord(key_c)) % 256)
            enc.append(enc_c)
        return base64.urlsafe_b64encode("".join(enc))

    @staticmethod
    def _decode(key, enc):
        dec = []
        enc = base64.urlsafe_b64decode(enc)
        for i in range(len(enc)):
            key_c = key[i % len(key)]
            dec_c = chr((256 + ord(enc[i]) - ord(key_c)) % 256)
            dec.append(dec_c)
        return "".join(dec)

    def decrypt(self, encryptedConfig):
        """!
        Entschlüsselt die übergebene HF-Config.
        @param encryptedConfig: Zu entschlüsselnde HF-Config.
        @return Entschlüsselte Heizdatei Konfiguration.
        """
        if not encryptedConfig.has_section('metainfo'):
            _logger.warning('Heatfile not valid, metainfo is missing!')  # Datei benötigt metainfo
            return None
        elif encryptedConfig.has_option('metainfo', 'encrypted'):   # Datei muss verschlüsselt sein
            # Verschlüsselte Daten aus metainfo lesen
            encrypted = encryptedConfig.get('metainfo', 'encrypted')
            encryptedConfig.remove_option('metainfo', 'encrypted')
            # Metainfo sichern
            metainfo = encryptedConfig.items('metainfo')
            # Daten entschlüsseln
            decrypted = self._decode(self._key, encrypted)
            # Daten in ein Dict umwandeln
            decrypted_dict = ast.literal_eval(decrypted)
            # Dict in Config umwandeln
            decryptedConfig = ConfigParser.ConfigParser()
            # Für jede entschlüsselte Section
            for section in decrypted_dict.keys():
                decryptedConfig.add_section(section)
                # Alle Key-Value Paare in die Config übernehmen
                for key in decrypted_dict[section].keys():
                    value = decrypted_dict[section][key]
                    decryptedConfig.set(section, key, value)
            # metainfo übernehmen
            decryptedConfig.add_section('metainfo')
            for name, value in metainfo:
                decryptedConfig.set('metainfo', name, value)
            # Config zurück geben
            return decryptedConfig
        else:
            _logger.warning('Heatfile has no encrypted data')
            return None

    def encrypt(self, decryptedConfig):
        """!
        Verschlüsselt die übergebene HF-Config.
        @param decryptedConfig: Zu verschlüsselnde HF-Config.
        @return Verschlüsselte Heizdatei Konfiguration.
        """
        if not decryptedConfig.has_section('metainfo'):  # Datei benötigt metainfo
            _logger.warning('Heatfile not valid, metainfo is missing!')
            return None
        elif decryptedConfig.has_option('metainfo', 'encrypted'):   # Datei darf nicht verschlüsselt sein
            _logger.warning('Heatfile is already encrypted')
            return None
        elif len(decryptedConfig.sections()) > 1:    # Datei benötigt mindestens metainfo und eine zusätzliche Section
            # Metainfo sichern
            metainfo = decryptedConfig.items('metainfo')

            decryptedConfig.remove_section('metainfo')  # metainfo wurde gelesen, daher nicht mehr benötigt
            hfconfigdict = {section: dict(decryptedConfig.items(section)) for section in decryptedConfig.sections()}

            # Daten verschlüsseln
            encrypted = self._encode(self._key, str(hfconfigdict))
            # Neue Config für versclüsselte Daten erstellen
            encryptedConfig = ConfigParser.ConfigParser()
            # Diese besteht nur aus der metainfo
            encryptedConfig.add_section('metainfo')
            for name, value in metainfo:
                encryptedConfig.set('metainfo', name, value)
            # Verschlüsselte Daten in die metainfo einfügen
            encryptedConfig.set('metainfo', 'encrypted', encrypted)

            return encryptedConfig
        else:
            _logger.warning('Heatfile has no data to encrypt.')
            return None


if __name__ == '__main__':
    """!
    Kommandozeilen Funktionen Ver- und Entschlüsseln von Heizdateien.
    """
    hilfe = "Verwendung: hfcrypto.py [Option] [Dateipfad]\n" \
            "Optionen:\n" \
            "  -e : Verschlüsseln der angegebenen Datei\n" \
            "  -d : Entschlüsseln der angegebenen Datei\n" \
            "Dateipfad: Diese Datei wird ver- oder entschlüsselt.\n" \
            "           Der Inhalt der Datei wird mit den ver- oder entschlüsselten Daten überschrieben."

    from logging.config import dictConfig
    import sys

    logging_config = dict(
        version=1,
        disable_existing_loggers=False,
        formatters={
            'f': {'format': '%(asctime)s %(name)-12s %(levelname)-8s [%(funcName)s():%(lineno)s]\t %(message)s'}
        },
        handlers={
            'h': {'class': 'logging.StreamHandler',
                  'formatter': 'f',
                  'level': logging.DEBUG}
        },
        root={
            'handlers': ['h'],
            'level': logging.DEBUG,
        },
    )
    dictConfig(logging_config)

    hfCrypto = HfCrypto()
    heatfileconfig = ConfigParser.ConfigParser()  # ConfigParser für ini-Dateien

    if len(sys.argv) != 3:
        print hilfe
    else:
        try:
            heatfileconfig.read(sys.argv[2])  # angegebene Datei lesen
        except:
            print "Datei '{}' kann nicht geöffnet werden.".format(sys.argv[2])
            print hilfe
        else:
            if sys.argv[1] == '-e':
                print 'Verschlüssle Datei: {}'.format(sys.argv[2])

                encryptedCfg = hfCrypto.encrypt(heatfileconfig)

                if encryptedCfg:
                    print {section: dict(encryptedCfg.items(section)) for section in encryptedCfg.sections()}

                    with open(sys.argv[2], 'w') as encryptedHeatFile:
                        encryptedCfg.write(encryptedHeatFile)
                else:
                    print "Datei '{}' konnte nicht verschlüsselt werden!".format(sys.argv[2])
            elif sys.argv[1] == '-d':
                print 'Entschlüssle Datei: {}'.format(sys.argv[2])

                decryptedCfg = hfCrypto.decrypt(heatfileconfig)
                if decryptedCfg:
                    print {section: dict(decryptedCfg.items(section)) for section in decryptedCfg.sections()}

                    with open(sys.argv[2], 'w') as decryptedHeatFile:
                        decryptedCfg.write(decryptedHeatFile)
                else:
                    print "Datei '{}' konnte nicht entschlüsselt werden!".format(sys.argv[2])
            else:
                print hilfe
